package cn.com.do1.component.bi.gateway.filter;

import cn.com.do1.component.bi.gateway.config.ReqDecryptAndRespEncryptConfig;
import cn.com.do1.component.bi.gateway.util.SpringContextUtils;
import cn.com.do1.component.bi.gateway.util.encrypt.AESEncryptUtil;
import cn.com.do1.component.bi.gateway.util.encrypt.RSAEncryptUtil;
import com.alibaba.fastjson.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.cloud.gateway.filter.GatewayFilter;
import org.springframework.cloud.gateway.filter.GatewayFilterChain;
import org.springframework.cloud.gateway.filter.factory.rewrite.CachedBodyOutputMessage;
import org.springframework.cloud.gateway.support.BodyInserterContext;
import org.springframework.context.annotation.DependsOn;
import org.springframework.core.Ordered;
import org.springframework.core.io.buffer.DataBuffer;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.MediaType;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.http.server.reactive.ServerHttpRequestDecorator;
import org.springframework.web.reactive.function.BodyInserter;
import org.springframework.web.reactive.function.BodyInserters;
import org.springframework.web.reactive.function.server.HandlerStrategies;
import org.springframework.web.reactive.function.server.ServerRequest;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

@DependsOn("reqDecryptAndRespEncryptConfig")
public class ReqDecryptFilter implements GatewayFilter, Ordered {

    private static final Logger logger = LoggerFactory.getLogger(ReqDecryptFilter.class);

    private int order;

    private static ReqDecryptAndRespEncryptConfig reqDecryptAndRespEncryptConfig = null;

    @Override
    public Mono<Void> filter(ServerWebExchange exchange, GatewayFilterChain chain) {
        if(reqDecryptAndRespEncryptConfig == null){
            reqDecryptAndRespEncryptConfig = SpringContextUtils.getBean("reqDecryptAndRespEncryptConfig");
        }
        ServerHttpRequest request = exchange.getRequest();
        if(!reqDecryptAndRespEncryptConfig.getReqDecrypt()){
            //关闭解密
            return chain.filter(exchange);
        }
        if(reqDecryptAndRespEncryptConfig.getWhiteUrl().size()>0){
            for(String url : reqDecryptAndRespEncryptConfig.getWhiteUrl()){
                if(request.getURI().getPath().startsWith(url)){
                    //白名单url直接跳过
                    return chain.filter(exchange);
                }
            }
        }
        HttpMethod requestMethod = request.getMethod();
        if (!HttpMethod.POST.equals(requestMethod) && !HttpMethod.PUT.equals(requestMethod)) {
            return chain.filter(exchange);
        }
        // 获取请求的Content-Type
        String contentType = request.getHeaders().getFirst(HttpHeaders.CONTENT_TYPE);
        if (!MediaType.APPLICATION_JSON_VALUE.equals(contentType)) {
            return chain.filter(exchange);
        }
        return readBody(exchange, chain);
    }
    ServerHttpRequestDecorator decorate(ServerWebExchange exchange, HttpHeaders headers, CachedBodyOutputMessage outputMessage) {
        return new ServerHttpRequestDecorator(exchange.getRequest()) {
            public HttpHeaders getHeaders() {
                long contentLength = headers.getContentLength();
                HttpHeaders httpHeaders = new HttpHeaders();
                httpHeaders.putAll(super.getHeaders());
                if (contentLength > 0L) {
                    httpHeaders.setContentLength(contentLength);
                } else {
                    httpHeaders.set("Transfer-Encoding", "chunked");
                }
                return httpHeaders;
            }
            public Flux<DataBuffer> getBody() {
                return outputMessage.getBody();
            }
        };
    }


    private Mono<Void> returnMononew(GatewayFilterChain chain,ServerWebExchange exchange){
        return chain.filter(exchange).then(Mono.fromRunnable(()->{
        }));
    }
    private Mono<Void> readBody(ServerWebExchange exchange, GatewayFilterChain chain) {
        //重新构造request，参考ModifyRequestBodyGatewayFilterFactory
        ServerRequest serverRequest = ServerRequest.create(exchange, HandlerStrategies.withDefaults().messageReaders());
        //重点
        Mono<String> modifiedBody = serverRequest.bodyToMono(String.class).flatMap(body -> {
                String newBody = decryptAES(body);
                return Mono.just(newBody);
        });
        BodyInserter bodyInserter = BodyInserters.fromPublisher(modifiedBody, String.class);
        HttpHeaders headers = new HttpHeaders();
        headers.putAll(exchange.getRequest().getHeaders());
        //猜测这个就是之前报400错误的元凶，之前修改了body但是没有重新写content length
        headers.remove("Content-Length");
        CachedBodyOutputMessage outputMessage = new CachedBodyOutputMessage(exchange, headers);
        return bodyInserter.insert(outputMessage, new BodyInserterContext()).then(Mono.defer(() -> {
            ServerHttpRequest decorator = this.decorate(exchange, headers, outputMessage);
            return returnMononew(chain, exchange.mutate().request(decorator).build());
        }));
    }
    private String decryptAES(String encryptedText) {
        // 解密逻辑
        JSONObject param = JSONObject.parseObject(encryptedText);
        if(param.containsKey("encrypted") && param.containsKey("requestData")){
            try {
                String clientKey = RSAEncryptUtil.decryptByPrivateKey(param.getString("encrypted"),reqDecryptAndRespEncryptConfig.getRsaPrivateKey());
                return AESEncryptUtil.decrypt(param.getString("requestData"), clientKey);
            } catch (Exception e) {
                logger.error("解密接口出现问题",e);
            }
        }else{
            logger.debug("需要解密内容为空");
        }
        return encryptedText;
    }

    @Override
    public int getOrder() {
        return this.order;
    }


    public ReqDecryptFilter(int order){
        this.order = order;
    }
}