package cn.com.do1.component.bi.gateway.filter;

import cn.com.do1.component.bi.gateway.config.ReqDecryptAndRespEncryptConfig;
import cn.com.do1.component.bi.gateway.util.RandomStringUtils;
import cn.com.do1.component.bi.gateway.util.SpringContextUtils;
import cn.com.do1.component.bi.gateway.util.encrypt.AESEncryptUtil;
import cn.com.do1.component.bi.gateway.util.encrypt.RSAEncryptUtil;
import com.alibaba.fastjson.JSONObject;
import org.reactivestreams.Publisher;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.cloud.gateway.filter.GatewayFilter;
import org.springframework.cloud.gateway.filter.GatewayFilterChain;
import org.springframework.core.Ordered;
import org.springframework.core.io.buffer.DataBuffer;
import org.springframework.core.io.buffer.DataBufferFactory;
import org.springframework.core.io.buffer.DataBufferUtils;
import org.springframework.core.io.buffer.DefaultDataBufferFactory;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.http.server.reactive.ServerHttpResponse;
import org.springframework.http.server.reactive.ServerHttpResponseDecorator;
import org.springframework.util.StringUtils;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import java.nio.charset.StandardCharsets;

public class RespEncryptFilter implements GatewayFilter, Ordered {
    private static final Logger logger = LoggerFactory.getLogger(RespEncryptFilter.class);

    private int order;
    private static ReqDecryptAndRespEncryptConfig reqDecryptAndRespEncryptConfig = null;

    @Override
    public Mono<Void> filter(ServerWebExchange exchange, GatewayFilterChain chain) {
        ServerHttpRequest request = exchange.getRequest();
        if(reqDecryptAndRespEncryptConfig == null){
            reqDecryptAndRespEncryptConfig = SpringContextUtils.getBean("reqDecryptAndRespEncryptConfig");
        }
        if(!reqDecryptAndRespEncryptConfig.getRespEncrypt()){
            //关闭解密
            return chain.filter(exchange);
        }
        if(reqDecryptAndRespEncryptConfig.getWhiteUrl().size()>0){
            for(String url : reqDecryptAndRespEncryptConfig.getWhiteUrl()){
                if(request.getURI().getPath().startsWith(url)){
                    //白名单url直接跳过
                    return chain.filter(exchange);
                }
            }
        }
        ServerHttpResponse originalResponse = exchange.getResponse();
        originalResponse.getHeaders().setContentType(MediaType.APPLICATION_JSON);
        DataBufferFactory bufferFactory = originalResponse.bufferFactory();
        ServerHttpResponseDecorator decoratedResponse = new ServerHttpResponseDecorator(originalResponse) {
            @Override
            public Mono<Void> writeWith(Publisher<? extends DataBuffer> body) {
                if (getStatusCode().equals(HttpStatus.OK) && body instanceof Flux) {
                    Flux<? extends DataBuffer> fluxBody = Flux.from(body);
                    return super.writeWith(fluxBody.buffer().map(dataBuffer -> {
                        DataBufferFactory dataBufferFactory = new DefaultDataBufferFactory();
                        DataBuffer join = dataBufferFactory.join(dataBuffer);
                        byte[] content = new byte[join.readableByteCount()];
                        join.read(content);
                        //释放掉内存
                        DataBufferUtils.release(join);
                        // 正常返回的数据
                        String rootData = new String(content, StandardCharsets.UTF_8);
                        logger.debug("--------"+request.getURI().getPath()+";"+rootData);
                        JSONObject result = JSONObject.parseObject(rootData);
                        if(!result.containsKey("code")){
                            return bufferFactory.wrap(rootData.getBytes());
                        }
                        JSONObject json = new JSONObject();
                        json.put("code", result.getInteger("code"));
                        json.put("message", result.getString("message"));
                        json.put("traceId", result.getString("traceId"));
                        try {
                            String data = result.getString("data");
                            if(result.containsKey("code") && 0 == result.getInteger("code") && StringUtils.hasLength(data)){
                                String aesKey = RandomStringUtils.generateRandomString(16);
                                String key = RSAEncryptUtil.encryptByPublicKeyToBase64(aesKey.getBytes(),reqDecryptAndRespEncryptConfig.getRsaPublicKey());
                                String encryptData = AESEncryptUtil.encrypt(data,aesKey);
                                JSONObject sonJson = new JSONObject();
                                sonJson.put("encrypted", key);
                                sonJson.put("responseData", encryptData);
                                json.put("data",sonJson);
                            }else{
                                json.put("data", null);
                            }
                        } catch (Exception e) {
                            logger.error("加密response失败",e);
                        }
                        // 加密后的数据返回给客户端
                        byte[] uppedContent = json.toJSONString().getBytes(StandardCharsets.UTF_8);
                        originalResponse.getHeaders().setContentLength(uppedContent.length);
                        return bufferFactory.wrap(uppedContent);
                    }));
                }
                return super.writeWith(body);
            }
        };
        return chain.filter(exchange.mutate().response(decoratedResponse).build());
    }
    @Override
    public int getOrder() {
        return this.order;
    }

    public RespEncryptFilter(int order){
        this.order = order;
    }
}